/*
 * Looking from component side, cpu on the right,
 *
 *           | |  ||||||||
 *           + G   + TR
 *           3 n   3 xx
 *             d
 * or,
 *           |    ||||||||||    ||
 *           +    TGR+
 *           5    xnx5
 *                 d
 *
 *  Power on
 *   AT           aaaaalo ?
 *   AT+CGMM      request model identification
 *   AT+CGMR      request revision identification
 *  WWW key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT+CKPD="91c91c911111110c"
 *   AT*ESVM=0
 *  Attachment key
 *   AT+CKPD="#11111110c"
 *  Email key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT+CKPD="8666*1111111*0c"
 *   AT*ESVM=0
 *  SMS key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT*ESVM=0
 *  Phonebook key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>s"
 *   AT*ESVM=0
 */

#include "standard.h"

STATIC byte
*functions[] = {
	"\003" "e"                , /* No         ^C */
	"\010" "c"                , /* <-         ^H */
	"\012" "s"                , /* Yes        NL */
	"<"    "<"                , /* Left       < */
	">"    ">"                , /* Right      > */
	"\000" "91c91c911111110c" , /* WWW        ? */
	"\000" "#11111110c"       , /* Attachment ? */
	"\000" "8666*1111111*0c"  , /* Email      ? spurious SMS first */
	"\000" "eseee<see<<s>>ss1", /* SMS        ? */
	"\000" "eseee<see<<s>>s"  , /* Phonebook  ? */
},
*keys[] = {
	"+&@/_%$____0",            /* _ marks non-existent keys */
	" -?!,.:_\"'___()1",
	"abc2",
	"def3",
	"ghi4",
	"jkl5",
	"mno6",
	"pqrs7",
	"tuv8",
	"wxyz9",
};

/* return ascii character or control character or 0.
 * 'str' trashed. */

static byte
characterize(byte *str)
{
	byte *end, **v;

	/* only AT+CKPD="foo"[,20] contains double quotes */

	if ((str = index(str, '"')) == NULL
	 || (end = index(++str, '"')) <= str)  /* includes NULL */
		return (0);

	*end++ = 0;               /* str now tidy, *end is \0 or comma */

	if (!strcmp(end, ",25"))
		return (0);           /* superfluous AT+CKPD="c",25 rejected */

	v = functions;
	do {
		if (!strcmp(*v + 1, str))
			return (**v);
	} while (++v < __ENDOF(functions));

	if (*end)
		return (str[0]);      /* digits have ,20 */

	if (str[1] == 'c')
		str += 2;             /* 1c333/0c1 oddities equal 333/1 */

	if (*str == '*')
		return ('_');           /* shift space */

	if (*str == '#')
		return (str[1] ? '*' : '#');   /* ## and # translate to * and # */

	if (isdigit(*str))
		return (keys[*str - '0'][strlen(str + 1)]);

	return (0);
}

/* chatboard_init(); needs to be called once */

extern byte chatboard_cnt, chatboard_buf[];

/* control characters returned are: 0 (no data) ^C NL BS */

byte
chatboard_getc()
{
	byte c = 0;
	byte n = chatboard_cnt;

	if (n && chatboard_buf[--n] == '\r') {

		/* after the CR, chatboard waits us to ack 'OK' */

		chatboard_buf[n] = 0;
		c = characterize(chatboard_buf);

		chatboard_cnt = 0;
		chatboard_OK();
	}
	return (c);
}

void
process_chatboard()
{
	byte c = chatboard_getc();

	if (c) {
		if (debug) {
			if (c == 0x03)
				print_string("^C\n");
			else
				putchar(c);  /* newline and backspace passed as-is */
		}
		keyboard(c);
	}
}

#if 0

void
chatboard_debug()
{
	static byte cb_rem, fill;

	while (cb_rem != chatboard_cnt) {
		byte c = chatboard_buf[cb_rem++];

		if (c == '\r') {
			chatboard_OK();
			eol();
		} else if (c >= ' ' && c < 0x7F)
			putchar(c);
		else {
			print_string("\\x");
			print_byte(c);
		}
	}
}
#endif

