
/* !DAO! extension to std/mic-e coordinates */

#include "standard.h"
#include "fix.h"

static byte
AZ_code(long *mm)
{
	word rem;

	EDIV(labs(*mm), 10, &rem);

	return (rem + '0');
}

void
add_daoism(struct coord *coord, byte **pp)
{
	byte *p = *pp;

	*pp += 5;

	p[0] = '!';
	p[1] = 'W'; /* WGS84, additional ascii digit, milliminutes */
	p[4] = '!';

	p[2] = AZ_code(&coord->lat);
	p[3] = AZ_code(&coord->lon);
}

/* b is 0...91 for 0...10 milliminutes */

static word
az_format_to_mm(word b)
{
	return (muldiv(b, 10, 91));
}

static void
nudge(long *mm, int smore)
{
	if (*mm < 0)
		smore = -smore; /* negative hemisphere, goes more negative */

	*mm += smore;
}

/*
 *  'p' is comment part from incoming packet payload.
 *  'coord' is already filled from std/mic-e data.
 *  Just WGS84 is used if found, other datums are ignored.
 */

void
any_daoism(struct coord *coord, byte *p, byte *end)
{
	byte la, lo;

	/* scan for !...! in the whole area */

	for (end -= 5 ; p <= end; ++p) {

		if (p[0] == '!'
		 && p[4] == '!') {

			switch (p[1]) {         /* datum character */

			default:
				if (isalnum(p[1]))  /* [0-9A-Za-z] is valid datum */
					return;
				break;

			case 'W':               /* WGS84, ascii digit for milliminute */
				if ((la = p[2] - '0') < 10
				 && (lo = p[3] - '0') < 10)
					goto add_millis;
				break;

			case 'w':               /* WGS84, centiminute in 91 steps */
				if ((la = p[2] - '!') < 91
				 && (lo = p[3] - '!') < 91) {
					la = az_format_to_mm(la);
					lo = az_format_to_mm(lo);
					goto add_millis;
				}
				break;
			}
		}
	}
	return;

add_millis:
	nudge(&coord->lat, la);
	nudge(&coord->lon, lo);
}

