
	include regs.inc

PIT_V equ 0A4h  ; V 52h

	;------------------------------------------
	section .bankf
	align 1

	global demod_lock, rssi_avg

rssi_avg     ds.w 1   ; accessed directly from afsk.s

demod_lock   ds.w 1
txproc_lock  ds.w 1

txproc_delay ds.w 1

	;----------------------------------------------------------------------
	section .bss
	align 1

pit_tick     ds.w 1   ; downcounts each second
pit_sec      ds.w 1   ; counts seconds. keep these 2 contiguous

	global time, _bolt, _lbolt

time         ds.w 2   ; freerunning milliseconds

_lbolt       ds.w 1
_bolt        ds.w 1   ; freerunning seconds, word and dword

_channel_utilization ds.w 2 ; dword

	;----------------------------------------------------------------------
	section .data
	align 1

	global _wallclock, _abbrclock

_wallclock           ; a "C" struct, keep wallclock.h in sync
_wall_year  dc.b 0
_abbrclock           ; also "C" struct, no year or second
_wall_mon   dc.b 1
_wall_day   dc.b 1   ; 1.1.2000
_wall_hour  dc.b 0
_wall_min   dc.b 0
_wall_sec   dc.b 0   ; 00:00:00

	;------------------------------------------
	section .inthand
	align 1

oops
	jmp stack_underflow       ; reboot

	; PIT hitting at 1024 Hz

PIT_int

	cps  # __lostack + 100    ; some margin
	blo  oops

	pshm d,e,x,y,z,k

	clrb
	tbek                           ; EXT ram

	; increment stuff and such
	; XXX if RSSI > 300 then used++  ---  try to handle collisions

	ldd  _afsk_dcd                 ; if softdcd, channel is used.
	ord  _force_dcd                ; or when tx_state > TRYING.
	cpd  # 18                      ; XXX tune for 1% with just noise
	blo  L100
	incw _channel_utilization + 2
	bne  L100
	incw _channel_utilization + 0
L100

	; binary time - not quite milliseconds, but close enough

	incw time + 2
	bne  L0
	incw time + 0    ; wrap after 4G
L0

	; wallclock base - exact seconds, needs help from mainline

	decw pit_tick    ; 1024 times a second
	bne  L1
	ldab # 4         ; 0x400 = 1024
	stab pit_tick    ; LSByte was already zero
	incw pit_sec     ; another second elapsed
	incw _bolt
	bne  L1
	incw _lbolt      ; MSWord
L1

	ldab # FF
	tbek             ; EXT F

	; rssi average

	ldd  rssi_avg
	asld
	asld
	asld
	subd rssi_avg   ; 7/8 old
	addd RJURR1     ; 1/8 new from RSSI adc
	lsrd
	lsrd
	lsrd
	std  rssi_avg

	; txproc timeout

	tstw txproc_delay
	beq  L3
	decw txproc_delay
L3

	orp  # 00060h          ; 2 to 3
	andp # 0FF3Fh          ; 3 to 1

	; demodulator

	decw demod_lock        ; try
	bne  L4                ; if not success
	ldab adc_buf_remptr + 1
	cmpb adc_buf_insptr + 1
	beq  L4

	jsr  afsk_rx_dsp       ; all regs free for use
	ldab # FF
	tbek
L4
	incw demod_lock        ; undo

	; txproc

	tstw txproc_delay
	bne  L6                ; not yet
	decw txproc_lock       ; try
	bne  L5                ; if not success

	clrb
	tbek
	tbxk
	tbyk                   ; banks as "C" likes them

	ldx  _txproc           ; variable in bank 0, pointing to routines in bank 3
	jsr  text_start,x      ; all regs free for use
	tde

	ldab # FF
	tbek
	ste  txproc_delay
L5
	incw txproc_lock       ; undo
L6

	pulm d,e,x,y,z,k
	rti


	;------------------------------------------
	section .text
	align 1

	;---------------------------------------
	global _init_PIT

_init_PIT

	pshm k,ccr
	orp  # 000E0h

	clrb
	tbek               ; external RAM

	ldd  # PIT_int
	std  PIT_V         ; vector 52h at address A4h

	ldd  # 1024
	std  pit_tick

	ldab # FF
	tbek               ; SFRs and internal ram

	ldd  # 0252h       ; IPL 2 IV 52h
	std  PICR
	ldd  # 8           ; 1024 Hz
	std  PITR

	pulm k,ccr
	rts

	;---------------------------------------
	global _channel_usage_percent

_channel_usage_percent

	xgex                ; 0,x == mark

	lded _channel_utilization
	pshm e,d

	clre
	clrd
	sted _channel_utilization

	lded time
	pshm e,d

	tsy                 ; 0,y == now, 4,y == n

	lde  0,y
	ldd  2,y
	subd 2,x
	sbce 0,x            ; elapsed in E:D
L102
	lsrw 4,y
	rorw 6,y            ; n >>= 1
	lsre
	rord                ; elapsed >>= 1
	tste
	bne  L102           ; while elapsed too wide

	lde  0,y
	ste  0,x
	lde  2,y
	ste  2,x            ; mark = now

	lde  6,y            ; n in E. elapsed in D, both scaled

	ais  #+8            ; stack purged

	ldx  # 100
	tstd
	beq  L103           ; huh? elapsed == 0
	xgdx                ; elapsed in x, 100 in D
	emul                ; E:D = 100 * n
	ediv                ; x = 100 * n / elapsed
L103
	xgdx                ; in D
	rts

	;---------------------------------------
	global _now, _future_msec, _future_sec
	global _time_past, _time_since
	global _delay_ms

_now              ; void now(stamp *t)
	xgex
	lded time
	ste  0,x
	std  2,x
	rts

_future_sec       ; void future_sec(stamp *t_E, word sec_D)
	xgex          ; pointer to stamp
	lde  # 1000   ; a second
	emul          ; ED is msec
	ste  0,x      ; msec MSWord
	bra  L30

_future_msec      ; void future_msec(stamp *t, word msec)
	xgex
	clrw 0,x      ; msec MSWord
L30
	std  2,x      ; msec LSWord
	lded time
	addd 2,x      ; msec
	std  2,x
	adce 0,x      ; ... MSWord
	ste  0,x
	rts

_time_since        ; word time_since(stamp *then)
	xgex
	lded time
	subd 2,x
	sbce 0,x       ; spent = now - then
	beq  L10
	clre
	clrd
	comd
L10
	rts

_time_past         ; bool time_past(stamp) XXX rethink this !
	pshm d,e
	tsx            ; E,D at x
	lded time
	subd 2,x
	sbce 0,x       ; ED negative if not yet passed that time
	come           ; positive  ...
	clrd
	asle           ; carry if passed
	rolb           ; FALSE or TRUE in D
	ais  # 4       ; purge argument dword
	rts

_delay_ms           ; not for delays over 30 seconds
	adde time + 2   ; target time
L7
	cpe  time + 2   ; XXX could/should wai ?
	bpl  L7
	rts

	;---------------------------------------
	global _read_rssi_avg

_read_rssi_avg
	pshm k
	ldab # FF
	tbek
	ldd  rssi_avg
	pulm k
	rts

	;---------------------------------------
	global _unleash_txproc, _hold_txproc, _okay_txproc

_unleash_txproc
	pshm k
	ldab # FF
	tbek
	ldd  # 1
	std  txproc_lock
	pulm k
	rts

_hold_txproc
	pshm k
	ldab # FF
	tbek
	decw txproc_lock
	pulm k
	rts

_okay_txproc
	pshm k
	ldab # FF
	tbek
	incw txproc_lock
	pulm k
	rts

	;----------------------------------------------------------------------

	global _wallclock_run, _wallclock_reset

_wallclock_reset         ; call before presetting wallclock

	lde  # 1024          ; "next second in a sec"
	clrd                 ; no cumulated seconds
	sted pit_tick        ; contiguous pit_tick and pit_sec
	rts

_wallclock_run           ; call at least twice a day

	pshm ccr
	orp  # 0040h         ; hold pit at 2

	ldd  pit_sec         ; cumulated seconds not yet reflected in wallclock
	clrw pit_sec         ; start again
	pulm ccr

	tstd
	bne  escalator_wc    ; interesting concept
	rts

escalator_wc

	addb _wall_sec
	adca # 0                  ; D has new seconds
	cpd  # 60
	bcs  L21                  ; if seconds still fit in current minute
	tde
L20
	ldab _wall_min
	incb                      ; another minute now
	cmpb # 60
	bcs  Lmi
	ldab _wall_hour
	incb
	cmpb # 24
	bcs  Lh
	ldab _wall_day
	incb                      ; another day now
	bsr  days_in_this_month   ; 28/30/31 to A
	cba                       ; A - B
	bcc  Ld                   ; if no overflow of month
	ldab _wall_mon
	incb
	cmpb # 13
	bcs  Lmo
	ldab _wall_year
	incb
	cmpb # 100
	bcs  Ly
	clrb
Ly	stab _wall_year
	ldab # 1
Lmo	stab _wall_mon
	ldab # 1
Ld	stab _wall_day
	clrb
Lh	stab _wall_hour
	clrb
Lmi	stab _wall_min

	sube # 60
	cpe  # 60
	bcc  L20
	ted
L21
	stab _wall_sec

	lded _abbrclock    ; month day hour and minute
	sted _aprs_ztamp   ; atomic update
	rts

	;---------------------------------------
	; 31,28,31,30,31,30,31,31,30,31,30,31
	;  1  2  3  4  5  6  7  8  9 10 11 12

days_in_this_month
	ldaa _wall_mon
	cmpa # 2       ; february ?
	beq  L31
	cmpa # 8       ; carry if month 1...7
	adca # -1      ; -= 1 if month 8...12
	anda # 1       ; 1 if odd
	adda # 30
	rts
L31
	ldaa _wall_year
	anda # 3
	suba # 1       ; carry if year 00 04 08 12 ...
	adca # 28      ; 29 if leap year
	rts            ; not caring if year 2100 is or is not leap year.

	;---------------------------------------
	end

