
/*
 *  stack of 24{128,256,512,1024}
 *  upto 2 Mbit total
 *
 *  single 241024 must be mapped as
 *  two separate chips in the header.
 *
 *  XXX store deltas
 */

#include "standard.h"
#include "fix.h"
#include "i2c.h"

struct record {
	struct coord coord;
};

#define MAGIC 0x1DE5

struct header {
	word ix;       /* must be 1st */
	word magic;
	byte chips[8]; /* (bytes / 256) | (address & 15) */
};

/*
 *  0x00 0x00                    for example, 8 pcs AT24LC256, size 0x8000
 *  0x1D 0xE5
 *  0x80 0x82 0x84 0x86 0x88 0x8A 0x8C 0x8E
 */

#define MINIMUM  (16384 / sizeof(struct record))
#define MAXIMUM  (65536 / sizeof(struct record))
#define RESERVED (2)             /* for header at chips[0] */

static
struct part {
	word addr;
	word slots;
}
	parts[8];

static word slots, ix;

/* straddling parts is not handled nor needed */

static word
partition(word ix, word *offset)
{
	struct part *p = parts;

	do {
		if (ix < p->slots) {
			*offset = ix * sizeof(struct record); /* MSbits are dontcare */
			return (p->addr);
		}
		ix -= p->slots;

	} while (++p < __ENDOF(parts));

	doh("trklog");
}

static bool
read(word ix, void *buf, word buflen)
{
	word offset, addr = partition(ix, &offset);

	return (i2c_wr(addr | NOSTOP, &offset, 2)
	     && i2c_rd(addr, buf, buflen));
}

static bool
write(word ix, void *buf, word buflen)
{
	word offset, addr = partition(ix, &offset);

	return (i2c_wr(addr | NOSTOP, &offset, 2)
	     && i2c_wr(NOSTART, buf, buflen));
}

static bool
init()
{
	struct header hdr;
	word probe;
	struct part *p;

	for (probe = 0xA0; probe <= 0xAE; probe += 2) {

		parts[0].addr  = probe;
		parts[0].slots = 1;

		if (read(0, &hdr, sizeof(hdr)) && hdr.magic == MAGIC) {
			byte *chips = hdr.chips;

			for (p = parts; p < __ENDOF(parts); ++p) {
				word addr, size, chip = *chips++;

				addr = (chip & 0x0F) | 0xA0;
				size = (chip & 0xF0) * 256 / sizeof(struct record);

				if (size == 0)
					size = MAXIMUM;

				if (size < MINIMUM || (addr & 1))
					goto reject;

				p->addr  = addr;
				p->slots = size;

				slots += size;
			}
			if (slots > RESERVED) {
				ix = hdr.ix;
				return (TRUE);
			}
		}
	reject:
		slots = 0;
	}
	return (FALSE);
}

void
trklog(struct coord *p)
{
	stamp timo;
	struct record record;

	if (slots || init()) {

		if (ix >= slots
		 || ix < RESERVED)
			ix = RESERVED;

		record.coord = *p;

		if (write(ix++, &record, sizeof(record))) {

			/* no point using delay_ms().
			 * just keep trying */

			now(&timo);
			do {
				if (write(0, &ix, sizeof(ix)))
					return;
			} while (time_since(&timo) <= 10);
		}
	}
	slots = 0; /* reinit next time */
}

