
; Verify packet header:
; only legal characters 0-9 A-Z (blank-padded to 6 bytes),
; control byte must exist (at least),
; digipeaters, if any, must have repeated/not-bits
; in proper sequence, any repeated before any
; not repeated.
; return TRUE/FALSE.
;
; command/response/reserved/ssid-bits are ignored.
; CRC must be checked elsewhere. Also, afsk.s already has
; some preliminary checks (length >= 15+2 and no residue bits)

	section .text
	align 1

	global _valid_ax25_upto_ctrl

_valid_ax25_upto_ctrl   ; bool v_a_u_c(byte *packet, word len_no_crc);

	subd # 15           ; dst, src and ctrl use 15 bytes
	bcs  invalid        ; with adjusted len, easier to scan digis

	xgex                ; packet in IX
	xgde                ; adjusted len in E

	bsr  scan_call      ; Last-bit returned in carry
	bcs  invalid        ; dst cannot be Last
	bsr  scan_call
	bcs  valid          ; if src was Last, success

	ldaa # 80h          ; catch repeated after not-repeated slots

loop_digis
	sube # 7            ; each digi uses 7 bytes
	bcs  invalid        ; seems truncated
	rola                ; previous repeated into carry
	ldaa 6,x            ; repeated into A.7, carry kept
	anda # 80h          ; zero condition if not repeated, carry kept
	bhi  invalid        ; no zero nor carry == this is repeated, previous not
	bsr  scan_call
	bcc  loop_digis     ; while not Last digi

valid
	ldd  # 1            ; good
	rts

invalid_pop
	ais  #+4            ; undo bsr scan_call
invalid
	clrd                ; not good
	rts

scan_call
	ldy  # 6            ; 6 characters in call

loop_call
	ldab 0,x
	aix  #+1
	lsrb                ; shift to ASCII (7-bit)
	bcs  invalid_pop    ; Last-bit only allowed at index 6

	; assume many letters, 1 or 2 digits, 1 or 0 spaces, is normal

	cmpb # 'Z'
	bhi  invalid_pop    ; above Z
	cmpb # 'A'
	bge  ok_char        ; [A-Z] (signed compare ok with 7-bit characters)
	cmpb # '9'
	bhi  invalid_pop    ; above 9
	cmpb # '0'
	bge  ok_char        ; [0-9]

	cpy  # 6
	beq  invalid_pop    ; call cannot be all blanks

test_blanks
	cmpb # ' '
	bne  invalid_pop    ; not a space either

	; after a space, only more space

	aiy  #-1
	beq  call_out       ; if end of call
	ldab 0,x
	aix  #+1
	lsrb                ; shift to ASCII (7-bit)
	bcc  test_blanks    ; if no Last-bit, keep testing for proper padding
	bra  invalid_pop    ; Last-bit only allowed at index 6

ok_char
	aiy  #-1
	bne  loop_call      ; while not at end of call

call_out
	ldab 0,x
	aix  #+1
	lsrb                ; Last in carry
	rts

